# Валидация команд и обработка ошибок

В этом документе описаны процессы валидации команд в Command Registry и способы обработки ошибок.

## Содержание

- [Процесс валидации](#процесс-валидации)
- [Типы валидаторов](#типы-валидаторов)
- [Режимы работы](#режимы-работы)
- [Типы ошибок](#типы-ошибок)
- [Обработка ошибок](#обработка-ошибок)
- [Автоматическое исправление](#автоматическое-исправление)

## Процесс валидации

Валидация команд выполняется перед их регистрацией в диспетчере. Процесс включает следующие шаги:

1. **Анализ исходного кода**: извлечение метаданных из докстрингов и аннотаций типов
2. **Проверка соответствия**: сравнение метаданных с формальными параметрами функции
3. **Принятие решения**: регистрация команды или вывод ошибок в зависимости от режима работы

```python
# Основной процесс валидации в методе register_command
def register_command(self, command_name: str, handler: Callable) -> bool:
    # Анализ обработчика
    metadata = self.analyze_handler(command_name, handler)
    
    # Валидация метаданных
    is_valid, errors = self.validate_handler(command_name, handler, metadata)
    
    # Вывод ошибок, если они есть
    if not is_valid:
        for error in errors:
            logger.error(f"{command_name}: {error}")
        
        # В строгом режиме прерываем регистрацию
        if self.strict and not self.auto_fix:
            return False
    
    # Регистрация команды
    self.dispatcher.register_handler(
        command=command_name,
        handler=handler,
        description=metadata["description"],
        summary=metadata["summary"],
        params=metadata["parameters"]
    )
    
    return True
```

## Типы валидаторов

Command Registry включает несколько типов валидаторов для разных аспектов проверки:

### 1. DocstringValidator

Проверяет соответствие докстрингов формальным параметрам функции:

- Наличие описания функции
- Описание всех параметров в секции Args
- Наличие секции Returns
- Соответствие имен параметров в докстринге и сигнатуре функции

```python
class DocstringValidator:
    def validate(self, handler: Callable, command_name: str, metadata: Dict[str, Any]) -> Tuple[bool, List[str]]:
        errors = []
        
        # Получаем формальные параметры функции
        sig = inspect.signature(handler)
        formal_params = list(sig.parameters.keys())
        
        # Парсим докстринг
        docstring = handler.__doc__ or ""
        parsed_doc = docstring_parser.parse(docstring)
        
        # Проверяем наличие описания функции
        if not parsed_doc.short_description and not parsed_doc.long_description:
            errors.append(f"Отсутствует описание функции")
        
        # Получаем параметры из докстринга
        doc_params = {param.arg_name: param for param in parsed_doc.params}
        
        # Проверяем, что все формальные параметры описаны в докстринге
        for param in formal_params:
            if param not in doc_params and param != 'params':
                errors.append(f"Параметр '{param}' не описан в докстринге функции")
        
        # Проверяем наличие returns в докстринге
        if not parsed_doc.returns:
            errors.append(f"Отсутствует описание возвращаемого значения в докстринге функции")
        
        return len(errors) == 0, errors
```

### 2. MetadataValidator

Проверяет соответствие метаданных команды её сигнатуре:

- Наличие всех обязательных параметров в метаданных
- Соответствие типов параметров в метаданных и аннотациях
- Отсутствие лишних параметров в метаданных

```python
class MetadataValidator:
    def validate(self, handler: Callable, command_name: str, metadata: Dict[str, Any]) -> Tuple[bool, List[str]]:
        errors = []
        
        # Получаем формальные параметры функции
        sig = inspect.signature(handler)
        
        # Проверяем соответствие метаданных и формальных параметров
        if "parameters" in metadata:
            for param_name, param_info in metadata["parameters"].items():
                # Проверяем наличие параметра в сигнатуре функции
                if param_name not in sig.parameters:
                    errors.append(f"Параметр '{param_name}' указан в метаданных, но отсутствует в сигнатуре функции")
            
            # Проверяем, что все обязательные параметры указаны в метаданных
            for param_name, param in sig.parameters.items():
                if param.default == inspect.Parameter.empty and param_name != 'self':
                    if param_name not in metadata["parameters"]:
                        errors.append(f"Обязательный параметр '{param_name}' не указан в метаданных")
        
        return len(errors) == 0, errors
```

## Режимы работы

Command Registry поддерживает несколько режимов валидации, которые определяют поведение при обнаружении ошибок:

### 1. Строгий режим (strict=True)

В строгом режиме система отказывается регистрировать команды с ошибками в документации:

- При обнаружении любой ошибки в документации команда не регистрируется
- Выводятся подробные сообщения об ошибках для каждой команды
- Рекомендуется использовать в CI/CD и продакшене для обеспечения качества кода

```python
# Пример использования строгого режима
registry = CommandRegistry(strict=True, auto_fix=False)
registry.register_all_commands()  # Команды с ошибками не будут зарегистрированы
```

### 2. Нестрогий режим (strict=False)

В нестрогом режиме система регистрирует все команды, даже с ошибками:

- При обнаружении ошибок выводятся предупреждения, но команда регистрируется
- Может привести к неточностям в документации API
- Полезен для быстрого прототипирования и отладки

```python
# Пример использования нестрогого режима
registry = CommandRegistry(strict=False, auto_fix=False)
registry.register_all_commands()  # Все команды будут зарегистрированы
```

### 3. Режим автоисправления (auto_fix=True)

В режиме автоисправления система пытается автоматически исправить ошибки:

- Недостающие метаданные извлекаются из аннотаций типов
- Улучшается документация на основе имеющейся информации
- Полезен при разработке, когда документация еще не полностью оформлена

```python
# Пример использования режима автоисправления
registry = CommandRegistry(strict=True, auto_fix=True)
registry.register_all_commands()  # Система попытается исправить ошибки
```

## Типы ошибок

Валидаторы могут обнаруживать различные типы ошибок:

### 1. Ошибки докстрингов

- **Отсутствие описания функции**: докстринг пуст или содержит только код
- **Отсутствие описания параметра**: параметр функции не описан в докстринге
- **Отсутствие описания возвращаемого значения**: нет секции Returns
- **Лишние параметры в докстринге**: в докстринге описаны параметры, которых нет в функции

### 2. Ошибки метаданных

- **Несоответствие типов**: тип в метаданных не соответствует аннотации типа
- **Отсутствие обязательного параметра**: обязательный параметр не описан в метаданных
- **Лишние параметры в метаданных**: в метаданных указаны параметры, которых нет в функции
- **Некорректное описание**: отсутствует или неполное описание команды или параметра

### 3. Ошибки аннотаций типов

- **Отсутствие аннотации типа**: параметр не имеет аннотации типа
- **Отсутствие аннотации возвращаемого значения**: функция не имеет аннотации возвращаемого значения
- **Несоответствие типов в аннотациях и метаданных**: тип в аннотации не соответствует типу в метаданных

## Обработка ошибок

При обнаружении ошибок Command Registry предоставляет подробную информацию для их исправления:

```
🚫 Ошибки в команде 'search_by_vector':
   - Параметр 'top_k' не описан в докстринге функции
   - Отсутствует описание возвращаемого значения в докстринге функции
   - Обязательный параметр 'vector' не указан в метаданных команды
```

В зависимости от режима работы, система может:

1. **Отказаться регистрировать команду** (strict=True, auto_fix=False)
2. **Зарегистрировать команду с предупреждениями** (strict=False)
3. **Попытаться исправить ошибки автоматически** (auto_fix=True)

### Рекомендуемый подход к обработке ошибок

1. **В разработке**:
   - Использовать auto_fix=True для автоматического исправления ошибок
   - Регулярно запускать валидацию в строгом режиме для выявления проблем

2. **В CI/CD**:
   - Использовать strict=True для блокирования PR с ошибками в документации
   - Добавить проверку валидации команд в процесс тестирования

3. **В продакшене**:
   - Всегда использовать strict=True для гарантии качества документации
   - Никогда не использовать auto_fix=True в продакшен-среде

## Автоматическое исправление

Когда включен режим auto_fix=True, система пытается исправить ошибки автоматически:

### 1. Дополнение метаданных

Если метаданные неполные, система дополняет их на основе:

- Аннотаций типов для определения типов параметров
- Сигнатуры функции для определения обязательных параметров
- Значений по умолчанию для определения default значений

```python
# Дополнение метаданных на основе аннотаций типов
type_hints = get_type_hints(handler)
for param_name, param_type in type_hints.items():
    if param_name == 'return':
        continue
        
    if param_name not in metadata["parameters"]:
        metadata["parameters"][param_name] = {
            "type": map_type_to_openapi(param_type),
            "description": f"Параметр {param_name}",
            "required": param_name in required_params
        }
```

### 2. Извлечение описаний из докстрингов

Если докстринги есть, но метаданные неполные, система извлекает описания из докстрингов:

```python
# Извлечение описаний из докстрингов
parsed_doc = docstring_parser.parse(handler.__doc__ or "")
for param in parsed_doc.params:
    param_name = param.arg_name
    if param_name in metadata["parameters"]:
        metadata["parameters"][param_name]["description"] = param.description
```

### 3. Генерация базовых описаний

Если описания отсутствуют, система генерирует базовые описания:

```python
# Генерация базовых описаний
if not metadata.get("description"):
    metadata["description"] = f"Команда {command_name}"

if not metadata.get("summary"):
    metadata["summary"] = command_name.replace("_", " ").title()
```

## Заключение

Система валидации Command Registry помогает обеспечить высокое качество документации API за счет строгой проверки соответствия докстрингов, метаданных и сигнатур функций. 

Рекомендуется использовать строгий режим валидации (strict=True) для гарантии качества документации, особенно в процессе CI/CD и в продакшен-среде. Режим автоисправления (auto_fix=True) полезен на ранних этапах разработки, когда документация еще не полностью оформлена. 